<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://designersuiteforwp.com
 * @since      1.0.0
 *
 * @package    Csd
 * @subpackage Csd/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Csd
 * @subpackage Csd/includes
 * @author     ORION <support@designersuiteforwp.com>
 */
class Csd {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Csd_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        if (defined('PLUGIN_NAME_VERSION')) {
            $this->version = PLUGIN_NAME_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'csd';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Csd_Loader. Orchestrates the hooks of the plugin.
     * - Csd_I18n. Defines internationalization functionality.
     * - Csd_Admin. Defines all hooks for the admin area.
     * - Csd_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-csd-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-csd-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-csd-admin.php';

        /**
         * The class responsible for defining all actions that occur in the admin area and related to products.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-csd-product.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-csd-public.php';

        $this->loader = new Csd_Loader();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Csd_I18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Csd_I18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Csd_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
        $this->loader->add_action('admin_menu', $plugin_admin, 'csd_add_submenu');
        $this->loader->add_action('admin_notices', $plugin_admin, 'csd_get_help_notices');
        $this->loader->add_action('admin_notices', $plugin_admin, 'notify_customization_page_missing');
        $this->loader->add_action('admin_notices', $plugin_admin, 'csd_get_max_input_vars_php_ini');
        $this->loader->add_action( 'admin_notices', $plugin_admin, 'notify_minmimum_required_parameters' );
        $this->loader->add_filter('upload_mimes', $plugin_admin, 'csd_add_custom_mime_types');
        $this->loader->add_action('init', $plugin_admin, 'csd_init_globals');
        // $this->loader->add_action('init', $plugin_admin, 'csd_get_updater');
        $this->loader->add_action( 'admin_init', $plugin_admin, 'csd_redirect' );

//newsletter subscription
         $this->loader->add_action('admin_notices', $plugin_admin, 'csd_get_subscription_notice');
        //subscription hooks
         $this->loader->add_action('wp_ajax_csd_subscribe', $plugin_admin, 'csd_subscribe');
         $this->loader->add_action('wp_ajax_nopriv_csd_subscribe', $plugin_admin, 'csd_subscribe');
        //hide subscription hooks
         $this->loader->add_action('wp_ajax_csd_hide_notice', $plugin_admin, 'csd_hide_notice');
         $this->loader->add_action('wp_ajax_nopriv_csd_hide_notice', $plugin_admin, 'csd_hide_notice');

        //pour faire apparaitre le bouton
         $this->loader->add_action('admin_notices', $plugin_admin, 'csd_get_license_activation_notice');

         $this->loader->add_action('wp_ajax_csd_activate_license', $plugin_admin, 'csd_activate_license');
         $this->loader->add_action('wp_ajax_nopriv_csd_activate_license', $plugin_admin, 'csd_activate_license');

        //Wordpress verification hook
        $this->loader->add_action('plugins_loaded', $plugin_admin, 'csd_verify_validity');

        $csd_config = new CSD_Config();
        //Allow us to hide the csd_data_upl meta from the meta list in the order details page
        $this->loader->add_action('init', $csd_config, 'csd_register_cpt_config');
        $this->loader->add_action('save_post_csd-config', $csd_config, 'csd_save_config');
        $this->loader->add_action('add_meta_boxes', $csd_config, 'csd_get_config_metabox');
        $this->loader->add_action('woocommerce_product_options_general_product_data', $csd_config, 'csd_get_product_config_selector');
        $this->loader->add_action('woocommerce_product_after_variable_attributes', $csd_config, 'csd_get_variation_product_config_selector', 10, 3);
        $this->loader->add_filter('get_user_option_meta-box-order_csd-config', $csd_config, 'csd_get_metabox_order');
        $this->loader->add_action('admin_action_csd_duplicate_config', $csd_config, 'csd_duplicate_config');
        $this->loader->add_filter('post_row_actions', $csd_config, 'csd_duplicate_post_link', 10, 2);
        $this->loader->add_action('woocommerce_save_product_variation', $csd_config, 'csd_save_variation_settings_fields');
        $this->loader->add_filter('get_user_option_meta-box-order_csd-config', $csd_config, 'csd_get_metabox_order');
        $this->loader->add_filter('screen_layout_columns', $csd_config, 'csd_get_layout_columns');
        $this->loader->add_filter( 'get_user_option_meta-box-order_csd-config', $csd_config, 'csd_config_order_metabox' );

        // hook clipart
        $csd_clipart = new CSD_Clipart();
        $this->loader->add_action('init', $csd_clipart, 'csd_register_cpt_cliparts');
        $this->loader->add_action('save_post_csd-cliparts', $csd_clipart, 'csd_save_cliparts');
        $this->loader->add_action('add_meta_boxes', $csd_clipart, 'csd_get_cliparts_metabox');
        $this->loader->add_filter('get_user_option_meta-box-order_csd-cliparts', $csd_clipart, 'csd_get_metabox_order');
        $this->loader->add_filter('screen_layout_columns', $csd_clipart, 'csd_get_layout_columns');

        $product_admin = new CSD_Product(false);
        $this->loader->add_filter('manage_edit-product_columns', $product_admin, 'csd_get_product_columns');
        $this->loader->add_action('woocommerce_product_options_inventory_product_data', $product_admin, 'csd_get_variable_product_details_location_notice');
        $this->loader->add_action('manage_product_posts_custom_column', $product_admin, 'csd_get_products_columns_values', 5, 2);
        $this->loader->add_action('save_post_product', $product_admin, 'csd_save_product_settings_fields');
        $this->loader->add_action('woocommerce_save_product_variation', $product_admin, 'csd_save_product_settings_fields');
        //custom line
        $this->loader->add_action('woocommerce_after_add_to_cart_button', $product_admin, 'csd_hide_cart_button');
        $this->loader->add_action('save_post_product', $product_admin, 'csd_save_config');
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Csd_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

        $this->loader->add_action('init', $plugin_public, 'csd_register_shortcodes');

        $this->loader->add_filter('query_vars', $plugin_public, 'csd_add_query_vars');
        $this->loader->add_filter('init', $plugin_public, 'csd_add_rewrite_rules', 99);

        //Variable filters
        $this->loader->add_action('init', $plugin_public, 'csd_set_variable_action_filters', 99);

        $this->loader->add_action('woocommerce_after_add_to_cart_button', $plugin_public, 'csd_get_customize_btn');
        $this->loader->add_filter('woocommerce_loop_add_to_cart_link', $plugin_public, 'csd_get_customize_btn_loop', 10, 2);

        $this->loader->add_action('wp_ajax_csd_save_user_choice', $plugin_public, 'csd_save_user_choice');
        $this->loader->add_action('wp_ajax_nopriv_csd_save_user_choice', $plugin_public, 'csd_save_user_choice');

        // Move user pictures upload from cookie in user account.
        $this->loader->add_action( 'user_register', $plugin_public, 'move_user_picture_uplaods', 10, 1 );

        $this->loader->add_action('wp_ajax_csd_add_custom_design_to_cart', $plugin_public, 'add_custom_design_to_cart_ajax');
        $this->loader->add_action('wp_ajax_nopriv_csd_add_custom_design_to_cart', $plugin_public, 'add_custom_design_to_cart_ajax');
        
        $this->loader->add_action('wp_ajax_csd_get_design_price', $plugin_public, 'csd_get_design_price');
        $this->loader->add_action('wp_ajax_nopriv_csd_get_design_price', $plugin_public, 'csd_get_design_price');

        $this->loader->add_action('wp_ajax_csd_generate_downloadable_file', $plugin_public, 'generate_downloadable_file');
        $this->loader->add_action('wp_ajax_nopriv_csd_generate_downloadable_file', $plugin_public, 'generate_downloadable_file');

        $this->loader->add_action('wp_ajax_csd_handle_picture_upload', $plugin_public, 'csd_handle_picture_upload');
        $this->loader->add_action('wp_ajax_nopriv_csd_handle_picture_upload', $plugin_public, 'csd_handle_picture_upload');

        // Handle delete user pictures upload.
        $this->loader->add_action( 'wp_ajax_csd_handle_delete_picture_upload', $plugin_public, 'csd_handle_delete_picture_upload' );
        $this->loader->add_action( 'wp_ajax_nopriv_csd_handle_delete_picture_upload', $plugin_public, 'csd_handle_delete_picture_upload' );

        //routes for generating design image
        /*$this->loader->add_action('wp_ajax_csd_generate_image', $plugin_public, 'csd_generate_image_ajax');
        $this->loader->add_action('wp_ajax_nopriv_csd_generate_image', $plugin_public, 'csd_generate_image_ajax');*/

        //save order metas
        $this->loader->add_action('woocommerce_new_order_item', $plugin_public, 'save_customized_item_meta', 10, 3);

        //Body class
        $this->loader->add_filter('body_class', $plugin_public, 'csd_add_class_to_body');

        //Shop loop item class
        $this->loader->add_filter('post_class', $plugin_public, 'csd_get_item_class', 10, 3);

        //get order meta
        $this->loader->add_action('woocommerce_after_order_itemmeta', $plugin_public, 'csd_get_order_custom_admin_data', 10, 3);

        //User account page
        $this->loader->add_action('woocommerce_order_item_meta_end', $plugin_public, 'csd_get_user_account_products_meta', 11, 4);

        //Send design as attachement.
        $this->loader->add_filter('woocommerce_email_attachments', $plugin_public, 'csd_add_order_design_to_mail', 10, 3);

        //calculate item price in cart
        $this->loader->add_action('woocommerce_before_calculate_totals', $plugin_public, 'csd_get_cart_item_price', 10);

        //Products
        $csd_product = new CSD_Product(false);
        $this->loader->add_filter('body_class', $csd_product, 'csd_get_custom_products_body_class', 10, 2);
        $this->loader->add_action('woocommerce_product_duplicate', $csd_product, 'csd_duplicate_product_metas', 10, 2);
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    csd_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }

}
